@@ -93,6 +93,7 @@
 #include "dfbvideosink.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 /* Debugging category */
 GST_DEBUG_CATEGORY_STATIC (dfbvideosink_debug);
@@ -143,6 +144,7 @@ static void gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
 static GstVideoSinkClass *parent_class = NULL;
 static GstBufferClass *surface_parent_class = NULL;
 
+#ifndef GST_DISABLE_GST_DEBUG
 static const char *
 gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
 {
@@ -191,6 +193,7 @@ gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
       return "UNKNOWN";
   }
 }
+#endif /* GST_DISABLE_GST_DEBUG */
 
 /* Creates miniobject and our internal surface */
 static GstDfbSurface *
@@ -267,8 +270,8 @@ gst_dfbvideosink_surface_create (GstDfbVideoSink * dfbvideosink, GstCaps * caps,
      or we are going to run into serious stride issues */
   if (GST_BUFFER_SIZE (surface) != size) {
     GST_WARNING_OBJECT (dfbvideosink, "DirectFB surface size (%dx%d=%d) "
-        "differs from GStreamer requested size %d", pitch, surface->height,
-        GST_BUFFER_SIZE (surface), size);
+        "differs from GStreamer requested size %u", pitch, surface->height,
+        GST_BUFFER_SIZE (surface), (guint) size);
     goto fallback;
   }
 
@@ -295,8 +298,8 @@ fallback:
     surface->surface->Release (surface->surface);
     surface->surface = NULL;
   }
-  GST_DEBUG_OBJECT (dfbvideosink, "allocating a buffer (%p) of %d bytes",
-      surface, size);
+  GST_DEBUG_OBJECT (dfbvideosink, "allocating a buffer (%p) of %u bytes",
+      surface, (guint) size);
 
   succeeded = TRUE;
 
@@ -332,8 +335,8 @@ gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
     gst_object_unref (dfbvideosink);
   }
 
-  GST_MINI_OBJECT_CLASS (surface_parent_class)->
-      finalize (GST_MINI_OBJECT (surface));
+  GST_MINI_OBJECT_CLASS (surface_parent_class)->finalize (GST_MINI_OBJECT
+      (surface));
 }
 
 static gpointer
@@ -691,7 +694,7 @@ gst_dfbvideosink_setup (GstDfbVideoSink * dfbvideosink)
           GstColorBalanceChannel *channel = NULL;
 
           GST_DEBUG_OBJECT (dfbvideosink, "adding %s as a colorbalance channel",
-              walk->data);
+              (const char *) walk->data);
 
           channel = g_object_new (GST_TYPE_COLOR_BALANCE_CHANNEL, NULL);
           channel->label = g_strdup (walk->data);
@@ -1538,7 +1541,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   if (mem_cpy) {
     IDirectFBSurface *dest = NULL, *surface = NULL;
-    gpointer data;
+    guint8 *data;
     gint dest_pitch, src_pitch, line;
     GstStructure *structure;
 
@@ -1585,7 +1588,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       dfbvideosink->layer->WaitForSync (dfbvideosink->layer);
     }
 
-    res = dest->Lock (dest, DSLF_WRITE, &data, &dest_pitch);
+    res = dest->Lock (dest, DSLF_WRITE, (void *) &data, &dest_pitch);
     if (res != DFB_OK) {
       GST_WARNING_OBJECT (dfbvideosink, "failed locking the external "
           "subsurface for writing");
@@ -1700,8 +1703,8 @@ gst_dfbvideosink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
 
   dfbvideosink = GST_DFBVIDEOSINK (bsink);
 
-  GST_DEBUG_OBJECT (dfbvideosink, "a buffer of %d bytes was requested "
-      "with caps %" GST_PTR_FORMAT " and offset %llu", size, caps, offset);
+  GST_LOG_OBJECT (dfbvideosink, "a buffer of %u bytes was requested with caps "
+      "%" GST_PTR_FORMAT " and offset %" G_GUINT64_FORMAT, size, caps, offset);
 
   if (G_UNLIKELY (!dfbvideosink->setup)) {
     GST_DEBUG_OBJECT (dfbvideosink, "we are not setup yet, can't allocate!");
@@ -1944,7 +1947,7 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   GstDfbVideoSink *dfbvideosink = GST_DFBVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  double x, y;
+  double x, y, old_x, old_y;
   GstPad *pad = NULL;
 
   src.w = GST_VIDEO_SINK_WIDTH (dfbvideosink);
@@ -1958,8 +1961,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   /* Our coordinates can be wrong here if we centered the video */
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
-    double old_x = x;
+  if (gst_structure_get_double (structure, "pointer_x", &old_x)) {
+    x = old_x;
 
     if (x >= result.x && x <= (result.x + result.w)) {
       x -= result.x;
@@ -1972,8 +1975,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
         "coordinate from %f to %f", old_x, x);
     gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
   }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
-    double old_y = y;
+  if (gst_structure_get_double (structure, "pointer_y", &old_y)) {
+    y = old_y;
 
     if (y >= result.y && y <= (result.y + result.h)) {
       y -= result.y;
