@@ -473,7 +473,8 @@ gst_schro_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
     if (GST_CLOCK_TIME_IS_VALID (base_video_decoder->last_sink_timestamp)) {
       base_video_decoder->current_frame->presentation_timestamp =
           base_video_decoder->last_sink_timestamp;
-      GST_DEBUG ("got timestamp %lld", base_video_decoder->last_sink_timestamp);
+      GST_DEBUG ("got timestamp %" G_GINT64_FORMAT,
+          base_video_decoder->last_sink_timestamp);
     } else if (base_video_decoder->last_sink_offset_end != -1) {
       GstVideoState *state;
 
@@ -545,6 +546,7 @@ gst_schro_dec_scan_for_sync (GstBaseVideoDecoder * base_video_decoder,
 {
   GstAdapter *adapter = base_video_decoder->input_adapter;
   int n_available;
+  int ret;
 
   n_available = gst_adapter_available (adapter) - offset;
 
@@ -556,10 +558,14 @@ gst_schro_dec_scan_for_sync (GstBaseVideoDecoder * base_video_decoder,
     }
   }
 
-  n_available -= 3;
+  n = MIN (n, n_available - 3);
 
-  return gst_adapter_masked_scan_uint32 (adapter, 0xffffffff, 0x42424344,
-      offset, MIN (n, n_available - 3));
+  ret = gst_adapter_masked_scan_uint32 (adapter, 0xffffffff, 0x42424344,
+      offset, n);
+  if (ret == -1) {
+    return n;
+  }
+  return ret;
 }
 
 
