@@ -50,10 +50,6 @@
 
 #include "gstdccpclientsink.h"
 #include "gstdccp.h"
-#include <string.h>             /* memset */
-#include <unistd.h>
-#include <arpa/inet.h>
-#include <fcntl.h>
 
 /* signals */
 enum
