@@ -21,25 +21,21 @@
 
 /**
  * SECTION:element-nuvdemux
+ * @see_also: mythtvsrc
  *
- * <refsect2>
- * <para>
  * Demuxes MythTVs NuppelVideo .nuv file into raw or compressed audio and/or
  * video streams.
- * </para>
- * <para>
+ * 
  * This element currently only supports pull-based scheduling.
- * </para>
+ * 
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc test.nuv ! nuvdemux name=demux  demux.audio_00 ! decodebin ! audioconvert ! audioresample ! autoaudiosink   demux.video_00 ! queue ! decodebin ! ffmpegcolorspace ! videoscale ! autovideosink
- * </programlisting>
- * Play (parse and decode) an .nuv file and try to output it to
+ * ]| Play (parse and decode) an .nuv file and try to output it to
  * an automatically detected soundcard and videosink. If the NUV file contains
  * compressed audio or video data, this will only work if you have the
  * right decoder elements/plugins installed.
- * </para>
  * </refsect2>
  *
  */
@@ -195,7 +191,8 @@ gst_nuv_demux_finalize (GObject * object)
   gst_nuv_demux_destoy_src_pad (nuv);
   gst_nuv_demux_reset (nuv);
   if (nuv->adapter != NULL) {
-    gst_object_unref (nuv->adapter);
+    g_object_unref (nuv->adapter);
+    nuv->adapter = NULL;
   }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -380,8 +377,9 @@ gst_nuv_demux_extended_header_load (GstNuvDemux * nuv,
 
   GST_DEBUG_OBJECT (nuv,
       "ex hdr: v=%d vffc=%4.4s afcc=%4.4s %dHz %dbits ach=%d acr=%d aq=%d"
-      "rtjpeg q=%d lf=%d lc=%d lavc br=%d qmin=%d qmax=%d maxqdiff=%d seekableoff=%lld keyfao=%lld",
-      h->i_version, (gchar *) & h->i_video_fcc, (gchar *) & h->i_audio_fcc,
+      "rtjpeg q=%d lf=%d lc=%d lavc br=%d qmin=%d qmax=%d maxqdiff=%d seekableoff=%"
+      G_GINT64_FORMAT " keyfao=%" G_GINT64_FORMAT, h->i_version,
+      (gchar *) & h->i_video_fcc, (gchar *) & h->i_audio_fcc,
       h->i_audio_sample_rate, h->i_audio_bits_per_sample, h->i_audio_channels,
       h->i_audio_compression_ratio, h->i_audio_quality, h->i_rtjpeg_quality,
       h->i_rtjpeg_luma_filter, h->i_rtjpeg_chroma_filter, h->i_lavc_bitrate,
@@ -804,7 +802,7 @@ gst_nuv_demux_sink_activate (GstPad * sinkpad)
   if (gst_pad_check_pull_range (sinkpad)) {
     nuv->mode = 0;
     if (nuv->adapter != NULL) {
-      gst_object_unref (nuv->adapter);
+      g_object_unref (nuv->adapter);
       nuv->adapter = NULL;
     }
     res = gst_pad_activate_pull (sinkpad, TRUE);
@@ -944,4 +942,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "nuvdemux",
     "Demuxes and muxes audio and video",
-    plugin_init, VERSION, "LGPL", "NuvDemux", "")
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
