@@ -380,6 +380,13 @@ gst_real_video_dec_setcaps (GstPad * pad, GstCaps * caps)
       "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
       "framerate", GST_TYPE_FRACTION, framerate_num, framerate_denom,
       "width", G_TYPE_INT, width, "height", G_TYPE_INT, height, NULL);
+
+  /* set PAR if one was specified in the sink caps */
+  if ((v = gst_structure_get_value (s, "pixel-aspect-ratio"))) {
+    gst_structure_set_value (gst_caps_get_structure (caps, 0),
+        "pixel-aspect-ratio", v);
+  }
+
   bres = gst_pad_set_caps (GST_PAD (dec->src), caps);
   gst_caps_unref (caps);
   if (!bres)
@@ -473,7 +480,17 @@ open_library (GstRealVideoDec * dec, GstRealVideoDecVersion version,
       gchar *codec = g_strconcat (split_path[i], "/", split_names[j], NULL);
 
       GST_DEBUG_OBJECT (dec, "trying %s", codec);
-      module = g_module_open (codec, G_MODULE_BIND_LAZY);
+      /* This is racy, but it doesn't matter here; would be nice if GModule
+       * gave us a GError instead of an error string, but it doesn't, so.. */
+      if (g_file_test (codec, G_FILE_TEST_EXISTS)) {
+        module = g_module_open (codec, G_MODULE_BIND_LAZY);
+        if (module == NULL) {
+          GST_ERROR_OBJECT (dec, "Could not open codec library '%s': %s",
+              codec, g_module_error ());
+        }
+      } else {
+        GST_LOG_OBJECT (dec, "%s does not exist", codec);
+      }
       g_free (codec);
       if (module)
         goto codec_search_done;
@@ -485,7 +502,7 @@ codec_search_done:
   g_strfreev (split_names);
 
   if (module == NULL)
-    goto could_not_open;
+    return FALSE;
 
   GST_DEBUG_OBJECT (dec, "module opened, finding symbols");
 
@@ -520,12 +537,6 @@ unknown_version:
     GST_ERROR_OBJECT (dec, "Cannot handle version %i.", version);
     return FALSE;
   }
-could_not_open:
-  {
-    GST_ERROR_OBJECT (dec, "Could not open library '%s' in '%s': %s", names,
-        path, g_module_error ());
-    return FALSE;
-  }
 could_not_load:
   {
     close_library (dec, lib);
