@@ -46,7 +46,6 @@ GST_ELEMENT_DETAILS ("Input selector",
     "Generic",
     "N-to-1 input stream selectoring",
     "Julien Moutte <julien@moutte.net>\n"
-    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>\n"
     "Jan Schmidt <thaytan@mad.scientist.com>\n"
     "Wim Taymans <wim.taymans@gmail.com>");
 
@@ -92,8 +91,8 @@ enum
 };
 static guint gst_input_selector_signals[LAST_SIGNAL] = { 0 };
 
-static gboolean gst_input_selector_is_active_sinkpad (GstInputSelector * sel,
-    GstPad * pad);
+static inline gboolean gst_input_selector_is_active_sinkpad (GstInputSelector *
+    sel, GstPad * pad);
 static GstPad *gst_input_selector_activate_sinkpad (GstInputSelector * sel,
     GstPad * pad);
 static GstPad *gst_input_selector_get_linked_pad (GstPad * pad,
@@ -149,7 +148,8 @@ static gint64 gst_selector_pad_get_running_time (GstSelectorPad * pad);
 static void gst_selector_pad_reset (GstSelectorPad * pad);
 static gboolean gst_selector_pad_event (GstPad * pad, GstEvent * event);
 static GstCaps *gst_selector_pad_getcaps (GstPad * pad);
-static GList *gst_selector_pad_get_linked_pads (GstPad * pad);
+static gboolean gst_selector_pad_acceptcaps (GstPad * pad, GstCaps * caps);
+static GstIterator *gst_selector_pad_iterate_linked_pads (GstPad * pad);
 static GstFlowReturn gst_selector_pad_chain (GstPad * pad, GstBuffer * buf);
 static GstFlowReturn gst_selector_pad_bufferalloc (GstPad * pad,
     guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
@@ -190,10 +190,8 @@ gst_selector_pad_class_init (GstSelectorPadClass * klass)
 
   gobject_class->finalize = gst_selector_pad_finalize;
 
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_selector_pad_get_property);
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_selector_pad_set_property);
+  gobject_class->get_property = gst_selector_pad_get_property;
+  gobject_class->set_property = gst_selector_pad_set_property;
 
   g_object_class_install_property (gobject_class, PROP_PAD_RUNNING_TIME,
       g_param_spec_int64 ("running-time", "Running time",
@@ -320,19 +318,22 @@ gst_selector_pad_reset (GstSelectorPad * pad)
 
 /* strictly get the linked pad from the sinkpad. If the pad is active we return
  * the srcpad else we return NULL */
-static GList *
-gst_selector_pad_get_linked_pads (GstPad * pad)
+static GstIterator *
+gst_selector_pad_iterate_linked_pads (GstPad * pad)
 {
+  GstInputSelector *sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
   GstPad *otherpad;
+  GstIterator *it;
 
   otherpad = gst_input_selector_get_linked_pad (pad, TRUE);
-  if (!otherpad)
-    return NULL;
+  it = gst_iterator_new_single (GST_TYPE_PAD, otherpad,
+      (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
 
-  /* need to drop the ref, internal linked pads is not MT safe */
-  gst_object_unref (otherpad);
+  if (otherpad)
+    gst_object_unref (otherpad);
+  gst_object_unref (sel);
 
-  return g_list_append (NULL, otherpad);
+  return it;
 }
 
 static gboolean
@@ -392,27 +393,36 @@ gst_selector_pad_event (GstPad * pad, GstEvent * event)
       gst_segment_set_newsegment_full (&selpad->segment, update,
           rate, arate, format, start, stop, time);
       GST_OBJECT_UNLOCK (selpad);
-      /* we are not going to forward the segment, mark the segment as
-       * pending */
-      forward = FALSE;
-      selpad->segment_pending = TRUE;
-      GST_INPUT_SELECTOR_UNLOCK (sel);
 
+      /* If we aren't forwarding the event (because the pad is not the
+       * active_sinkpad, and select_all is not set, then set the flag on the
+       * that says a segment needs sending if/when that pad is activated.
+       * For all other cases, we send the event immediately, which makes
+       * sparse streams and other segment updates work correctly downstream.
+       */
+      if (!forward)
+        selpad->segment_pending = TRUE;
+
+      GST_INPUT_SELECTOR_UNLOCK (sel);
       break;
     }
     case GST_EVENT_TAG:
     {
-      GstTagList *tags;
+      GstTagList *tags, *oldtags, *newtags;
+
+      gst_event_parse_tag (event, &tags);
 
       GST_OBJECT_LOCK (selpad);
-      if (selpad->tags)
-        gst_tag_list_free (selpad->tags);
-      gst_event_parse_tag (event, &tags);
-      if (tags)
-        tags = gst_tag_list_copy (tags);
-      selpad->tags = tags;
-      GST_DEBUG_OBJECT (pad, "received tags %" GST_PTR_FORMAT, selpad->tags);
+      oldtags = selpad->tags;
+
+      newtags = gst_tag_list_merge (oldtags, tags, GST_TAG_MERGE_REPLACE);
+      selpad->tags = newtags;
+      if (oldtags)
+        gst_tag_list_free (oldtags);
+      GST_DEBUG_OBJECT (pad, "received tags %" GST_PTR_FORMAT, newtags);
       GST_OBJECT_UNLOCK (selpad);
+
+      g_object_notify (G_OBJECT (selpad), "tags");
       break;
     }
     case GST_EVENT_EOS:
@@ -455,6 +465,21 @@ gst_selector_pad_getcaps (GstPad * pad)
   return caps;
 }
 
+static gboolean
+gst_selector_pad_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  GstInputSelector *sel;
+  gboolean res;
+
+  sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
+
+  GST_DEBUG_OBJECT (sel, "Checking acceptcaps of srcpad peer");
+  res = gst_pad_peer_accept_caps (sel->srcpad, caps);
+  gst_object_unref (sel);
+
+  return res;
+}
+
 static GstFlowReturn
 gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf)
@@ -532,9 +557,10 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   GstPad *active_sinkpad;
   GstPad *prev_active_sinkpad;
   GstSelectorPad *selpad;
-  GstClockTime end_time, duration;
+  GstClockTime start_time;
   GstSegment *seg;
   GstEvent *close_event = NULL, *start_event = NULL;
+  GstCaps *caps;
 
   sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
   selpad = GST_SELECTOR_PAD_CAST (pad);
@@ -551,16 +577,16 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   active_sinkpad = gst_input_selector_activate_sinkpad (sel, pad);
 
   /* update the segment on the srcpad */
-  end_time = GST_BUFFER_TIMESTAMP (buf);
-  if (GST_CLOCK_TIME_IS_VALID (end_time)) {
-    duration = GST_BUFFER_DURATION (buf);
-    if (GST_CLOCK_TIME_IS_VALID (duration))
-      end_time += duration;
-    GST_DEBUG_OBJECT (pad, "received end time %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (end_time));
+  start_time = GST_BUFFER_TIMESTAMP (buf);
+  if (GST_CLOCK_TIME_IS_VALID (start_time)) {
+    GST_DEBUG_OBJECT (pad, "received start time %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (start_time));
+    if (GST_BUFFER_DURATION_IS_VALID (buf))
+      GST_DEBUG_OBJECT (pad, "received end time %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (start_time + GST_BUFFER_DURATION (buf)));
 
     GST_OBJECT_LOCK (pad);
-    gst_segment_set_last_stop (seg, seg->format, end_time);
+    gst_segment_set_last_stop (seg, seg->format, start_time);
     GST_OBJECT_UNLOCK (pad);
   }
 
@@ -572,7 +598,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
     GstSegment *cseg = &sel->segment;
 
     GST_DEBUG_OBJECT (sel,
-        "pushing NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "pushing close NEWSEGMENT update %d, rate %lf, applied rate %lf, "
         "format %d, "
         "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
         G_GINT64_FORMAT, TRUE, cseg->rate, cseg->applied_rate, cseg->format,
@@ -587,7 +613,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   /* if we have a pending segment, push it out now */
   if (G_UNLIKELY (selpad->segment_pending)) {
     GST_DEBUG_OBJECT (pad,
-        "pushing NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "pushing pending NEWSEGMENT update %d, rate %lf, applied rate %lf, "
         "format %d, "
         "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
         G_GINT64_FORMAT, FALSE, seg->rate, seg->applied_rate, seg->format,
@@ -621,6 +647,11 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   GST_DEBUG_OBJECT (pad, "Forwarding buffer %p from pad %s:%s", buf,
       GST_DEBUG_PAD_NAME (pad));
 
+  if ((caps = GST_BUFFER_CAPS (buf))) {
+    if (GST_PAD_CAPS (sel->srcpad) != caps)
+      gst_pad_set_caps (sel->srcpad, caps);
+  }
+
   res = gst_pad_push (sel->srcpad, buf);
 
 done:
@@ -732,10 +763,8 @@ gst_input_selector_class_init (GstInputSelectorClass * klass)
 
   gobject_class->dispose = gst_input_selector_dispose;
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_input_selector_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_input_selector_get_property);
+  gobject_class->set_property = gst_input_selector_set_property;
+  gobject_class->get_property = gst_input_selector_get_property;
 
   g_object_class_install_property (gobject_class, PROP_N_PADS,
       g_param_spec_uint ("n-pads", "Number of Pads",
@@ -827,8 +856,8 @@ static void
 gst_input_selector_init (GstInputSelector * sel)
 {
   sel->srcpad = gst_pad_new ("src", GST_PAD_SRC);
-  gst_pad_set_internal_link_function (sel->srcpad,
-      GST_DEBUG_FUNCPTR (gst_selector_pad_get_linked_pads));
+  gst_pad_set_iterate_internal_links_function (sel->srcpad,
+      GST_DEBUG_FUNCPTR (gst_selector_pad_iterate_linked_pads));
   gst_pad_set_getcaps_function (sel->srcpad,
       GST_DEBUG_FUNCPTR (gst_input_selector_getcaps));
   gst_pad_set_query_function (sel->srcpad,
@@ -876,7 +905,10 @@ gst_input_selector_dispose (GObject * object)
 static gint64
 gst_segment_get_timestamp (GstSegment * segment, gint64 running_time)
 {
-  return (running_time - segment->accum) * segment->abs_rate + segment->start;
+  if (running_time <= segment->accum)
+    return segment->start;
+  else
+    return (running_time - segment->accum) * segment->abs_rate + segment->start;
 }
 
 static void
@@ -1173,14 +1205,11 @@ gst_input_selector_getcaps (GstPad * pad)
 }
 
 /* check if the pad is the active sinkpad */
-static gboolean
+static inline gboolean
 gst_input_selector_is_active_sinkpad (GstInputSelector * sel, GstPad * pad)
 {
-  GstSelectorPad *selpad;
   gboolean res;
 
-  selpad = GST_SELECTOR_PAD_CAST (pad);
-
   GST_INPUT_SELECTOR_LOCK (sel);
   res = (pad == sel->active_sinkpad);
   GST_INPUT_SELECTOR_UNLOCK (sel);
@@ -1237,10 +1266,12 @@ gst_input_selector_request_new_pad (GstElement * element,
       GST_DEBUG_FUNCPTR (gst_selector_pad_event));
   gst_pad_set_getcaps_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_getcaps));
+  gst_pad_set_acceptcaps_function (sinkpad,
+      GST_DEBUG_FUNCPTR (gst_selector_pad_acceptcaps));
   gst_pad_set_chain_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_chain));
-  gst_pad_set_internal_link_function (sinkpad,
-      GST_DEBUG_FUNCPTR (gst_selector_pad_get_linked_pads));
+  gst_pad_set_iterate_internal_links_function (sinkpad,
+      GST_DEBUG_FUNCPTR (gst_selector_pad_iterate_linked_pads));
   gst_pad_set_bufferalloc_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_selector_pad_bufferalloc));
 
@@ -1408,6 +1439,7 @@ gst_input_selector_check_eos (GstElement * selector)
         if (!pad->eos) {
           done = TRUE;
         }
+        gst_object_unref (pad);
         break;
       case GST_ITERATOR_RESYNC:
         gst_iterator_resync (it);
