@@ -2,7 +2,7 @@
  * Copyright (C)  2007 Sebastien Moutte <sebastien@moutte.net>
  * Copyright (C)  2009 Julien Isorce <julien.isorce@gmail.com>
  *
- * gstdshowvideosrc.c: 
+ * gstdshowvideosrc.c:
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,25 +37,17 @@ GST_ELEMENT_DETAILS ("DirectShow video capture source",
 GST_DEBUG_CATEGORY_STATIC (dshowvideosrc_debug);
 #define GST_CAT_DEFAULT dshowvideosrc_debug
 
-const GUID MEDIASUBTYPE_I420
-    = { 0x30323449, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xAA, 0x00, 0x38, 0x9B,
-    0x71}
-};
-
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (
-        GST_VIDEO_CAPS_BGR ";"
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_BGR ";"
         GST_VIDEO_CAPS_YUV ("{ I420 }") ";"
         "video/x-dv,"
         "systemstream = (boolean) FALSE,"
         "width = (int) [ 1, MAX ],"
         "height = (int) [ 1, MAX ],"
         "framerate = (fraction) [ 0, MAX ],"
-        "format = (fourcc) dvsd;"
-        "video/x-dv,"
-        "systemstream = (boolean) TRUE")
+        "format = (fourcc) dvsd;" "video/x-dv," "systemstream = (boolean) TRUE")
     );
 
 static void gst_dshowvideosrc_init_interfaces (GType type);
@@ -106,9 +98,11 @@ static GstFlowReturn gst_dshowvideosrc_create (GstPushSrc * psrc,
 
 /*utils*/
 static GstCaps *gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc *
-    src, IPin * pin, IAMStreamConfig * streamcaps);
-static gboolean gst_dshowvideosrc_push_buffer (byte * buffer, long size,
-    gpointer src_object, UINT64 start, UINT64 stop);
+    src, IPin * pin);
+static GstCaps *gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc *
+    src, IPin * pin);
+static gboolean gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size,
+    gpointer src_object, GstClockTime duration);
 
 static void
 gst_dshowvideosrc_init_interfaces (GType type)
@@ -179,14 +173,14 @@ gst_dshowvideosrc_class_init (GstDshowVideoSrcClass * klass)
   g_object_class_install_property
       (gobject_class, PROP_DEVICE,
       g_param_spec_string ("device", "Device",
-          "Directshow device path (@..classID/name)", NULL, 
-          static_cast<GParamFlags>(G_PARAM_READWRITE)));
+          "Directshow device path (@..classID/name)", NULL,
+          static_cast < GParamFlags > (G_PARAM_READWRITE)));
 
   g_object_class_install_property
       (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
-          "Human-readable name of the sound device", NULL, 
-          static_cast<GParamFlags>(G_PARAM_READWRITE)));
+          "Human-readable name of the sound device", NULL,
+          static_cast < GParamFlags > (G_PARAM_READWRITE)));
 
   GST_DEBUG_CATEGORY_INIT (dshowvideosrc_debug, "dshowvideosrc", 0,
       "Directshow video source");
@@ -203,7 +197,6 @@ gst_dshowvideosrc_init (GstDshowVideoSrc * src, GstDshowVideoSrcClass * klass)
   src->media_filter = NULL;
   src->filter_graph = NULL;
   src->caps = NULL;
-  src->video_defaults = NULL;
   src->pins_mediatypes = NULL;
   src->is_rgb = FALSE;
 
@@ -237,12 +230,16 @@ gst_dshowvideosrc_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
   }
 
   if (res != -1) {
-    GList *type_video_default = g_list_nth (src->video_defaults, res);
-    if (type_video_default) {
-      GstCaptureVideoDefault *video_default = (GstCaptureVideoDefault *) type_video_default->data;
-      gst_structure_fixate_field_nearest_int (structure, "width", video_default->defaultWidth);
-      gst_structure_fixate_field_nearest_int (structure, "height", video_default->defaultHeight);
-      gst_structure_fixate_field_nearest_fraction (structure, "framerate", video_default->defaultFPS, 1);
+    GList *type_pin_mediatype = g_list_nth (src->pins_mediatypes, res);
+    if (type_pin_mediatype) {
+      GstCapturePinMediaType *pin_mediatype =
+          (GstCapturePinMediaType *) type_pin_mediatype->data;
+      gst_structure_fixate_field_nearest_int (structure, "width",
+          pin_mediatype->defaultWidth);
+      gst_structure_fixate_field_nearest_int (structure, "height",
+          pin_mediatype->defaultHeight);
+      gst_structure_fixate_field_nearest_fraction (structure, "framerate",
+          pin_mediatype->defaultFPS, 1);
     }
   }
 }
@@ -267,11 +264,6 @@ gst_dshowvideosrc_dispose (GObject * gobject)
     src->caps = NULL;
   }
 
-  if (src->video_defaults) {
-    g_list_free (src->video_defaults);
-    src->video_defaults = NULL;
-  }
-
   if (src->pins_mediatypes) {
     gst_dshow_free_pins_mediatypes (src->pins_mediatypes);
     src->pins_mediatypes = NULL;
@@ -279,7 +271,7 @@ gst_dshowvideosrc_dispose (GObject * gobject)
 
   /* clean dshow */
   if (src->video_cap_filter) {
-    src->video_cap_filter->Release();
+    src->video_cap_filter->Release ();
     src->video_cap_filter = NULL;
   }
 
@@ -360,35 +352,34 @@ gst_dshowvideosrc_get_device_name_values (GstDshowVideoSrc * src)
   ULONG fetched;
 
   hres = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
-      IID_ICreateDevEnum, (LPVOID *) &devices_enum);
+      IID_ICreateDevEnum, (LPVOID *) & devices_enum);
   if (hres != S_OK) {
     GST_ERROR ("Can't create system device enumerator (error=0x%x)", hres);
     array = NULL;
     goto clean;
   }
 
-  hres = devices_enum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, 
-    &moniker_enum, 0);
+  hres = devices_enum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory,
+      &moniker_enum, 0);
   if (hres != S_OK || !moniker_enum) {
     GST_ERROR ("Can't get enumeration of video devices (error=0x%x)", hres);
     array = NULL;
     goto clean;
   }
 
-  moniker_enum->Reset();
+  moniker_enum->Reset ();
 
-  while (hres = moniker_enum->Next(1, &moniker, &fetched),
-      hres == S_OK) {
+  while (hres = moniker_enum->Next (1, &moniker, &fetched), hres == S_OK) {
     IPropertyBag *property_bag = NULL;
 
     hres =
-      moniker->BindToStorage(NULL, NULL, IID_IPropertyBag,
-        (LPVOID *) &property_bag);
+        moniker->BindToStorage (NULL, NULL, IID_IPropertyBag,
+        (LPVOID *) & property_bag);
     if (SUCCEEDED (hres) && property_bag) {
       VARIANT varFriendlyName;
 
       VariantInit (&varFriendlyName);
-      hres = property_bag->Read(L"FriendlyName", &varFriendlyName, NULL);
+      hres = property_bag->Read (L"FriendlyName", &varFriendlyName, NULL);
       if (hres == S_OK && varFriendlyName.bstrVal) {
         gchar *friendly_name =
             g_utf16_to_utf8 ((const gunichar2 *) varFriendlyName.bstrVal,
@@ -402,17 +393,17 @@ gst_dshowvideosrc_get_device_name_values (GstDshowVideoSrc * src)
         g_free (friendly_name);
         SysFreeString (varFriendlyName.bstrVal);
       }
-      property_bag->Release();
+      property_bag->Release ();
     }
-    moniker->Release();
+    moniker->Release ();
   }
 
 clean:
   if (moniker_enum)
-    moniker_enum->Release();
+    moniker_enum->Release ();
 
   if (devices_enum)
-    devices_enum->Release();
+    devices_enum->Release ();
 
   return array;
 }
@@ -488,31 +479,34 @@ gst_dshowvideosrc_get_caps (GstBaseSrc * basesrc)
   GstDshowVideoSrc *src = GST_DSHOWVIDEOSRC (basesrc);
   gunichar2 *unidevice = NULL;
 
-  if (src->device) {
-    g_free (src->device);
-    src->device = NULL;
+  if (src->caps) {
+    return gst_caps_ref (src->caps);
   }
 
-  src->device =
-      gst_dshow_getdevice_from_devicename (&CLSID_VideoInputDeviceCategory,
-      &src->device_name);
   if (!src->device) {
-    GST_ERROR ("No video device found.");
-    return NULL;
+    src->device =
+        gst_dshow_getdevice_from_devicename (&CLSID_VideoInputDeviceCategory,
+        &src->device_name);
+    if (!src->device) {
+      GST_ERROR ("No video device found.");
+      return NULL;
+    }
   }
+
   unidevice =
       g_utf8_to_utf16 (src->device, strlen (src->device), NULL, NULL, NULL);
 
   if (!src->video_cap_filter) {
     hres = CreateBindCtx (0, &lpbc);
     if (SUCCEEDED (hres)) {
-      hres = MkParseDisplayName (lpbc, (LPCOLESTR) unidevice, &dwEaten, &videom);
+      hres =
+          MkParseDisplayName (lpbc, (LPCOLESTR) unidevice, &dwEaten, &videom);
       if (SUCCEEDED (hres)) {
-        hres = videom->BindToObject(lpbc, NULL, IID_IBaseFilter, 
-          (LPVOID *) &src->video_cap_filter);
-        videom->Release();
+        hres = videom->BindToObject (lpbc, NULL, IID_IBaseFilter,
+            (LPVOID *) & src->video_cap_filter);
+        videom->Release ();
       }
-      lpbc->Release();
+      lpbc->Release ();
     }
   }
 
@@ -526,45 +520,42 @@ gst_dshowvideosrc_get_caps (GstBaseSrc * basesrc)
     IEnumPins *enumpins = NULL;
     HRESULT hres;
 
-    hres = src->video_cap_filter->EnumPins(&enumpins);
+    hres = src->video_cap_filter->EnumPins (&enumpins);
     if (SUCCEEDED (hres)) {
-      while (enumpins->Next(1, &capture_pin, NULL) == S_OK) {
+      while (enumpins->Next (1, &capture_pin, NULL) == S_OK) {
         IKsPropertySet *pKs = NULL;
-        hres = capture_pin->QueryInterface(IID_IKsPropertySet, (LPVOID *) &pKs);
+        hres =
+            capture_pin->QueryInterface (IID_IKsPropertySet, (LPVOID *) & pKs);
         if (SUCCEEDED (hres) && pKs) {
           DWORD cbReturned;
           GUID pin_category;
           RPC_STATUS rpcstatus;
 
           hres =
-              pKs->Get(AMPROPSETID_Pin,
+              pKs->Get (AMPROPSETID_Pin,
               AMPROPERTY_PIN_CATEGORY, NULL, 0, &pin_category, sizeof (GUID),
               &cbReturned);
 
           /* we only want capture pins */
-          if (UuidCompare (&pin_category, (UUID *) &PIN_CATEGORY_CAPTURE,
+          if (UuidCompare (&pin_category, (UUID *) & PIN_CATEGORY_CAPTURE,
                   &rpcstatus) == 0) {
-            IAMStreamConfig *streamcaps = NULL;
-
-            if (SUCCEEDED (capture_pin->QueryInterface(
-                        IID_IAMStreamConfig, (LPVOID *) &streamcaps))) {
+            {
               GstCaps *caps =
-                  gst_dshowvideosrc_getcaps_from_streamcaps (src, capture_pin,
-                  streamcaps);
-
+                  gst_dshowvideosrc_getcaps_from_streamcaps (src, capture_pin);
               if (caps) {
                 gst_caps_append (src->caps, caps);
+              } else {
+                caps = gst_dshowvideosrc_getcaps_from_enum_mediatypes (src, capture_pin);
+                if (caps)
+                  gst_caps_append (src->caps, caps);
               }
-              streamcaps->Release();
             }
           }
-
-          pKs->Release();
+          pKs->Release ();
         }
-
-        capture_pin->Release();
+        capture_pin->Release ();
       }
-      enumpins->Release();
+      enumpins->Release ();
     }
   }
 
@@ -573,7 +564,6 @@ gst_dshowvideosrc_get_caps (GstBaseSrc * basesrc)
   }
 
   if (src->caps) {
-    GST_LOG ("getcaps returned %s", gst_caps_to_string (src->caps));
     return gst_caps_ref (src->caps);
   }
 
@@ -593,7 +583,7 @@ gst_dshowvideosrc_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       if (src->media_filter)
-        hres = src->media_filter->Run(0);
+        hres = src->media_filter->Run (0);
       if (hres != S_OK) {
         GST_ERROR ("Can't RUN the directshow capture graph (error=0x%x)", hres);
         return GST_STATE_CHANGE_FAILURE;
@@ -601,7 +591,7 @@ gst_dshowvideosrc_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       if (src->media_filter)
-        hres = src->media_filter->Stop();
+        hres = src->media_filter->Stop ();
       if (hres != S_OK) {
         GST_ERROR ("Can't STOP the directshow capture graph (error=%d)", hres);
         return GST_STATE_CHANGE_FAILURE;
@@ -625,27 +615,32 @@ gst_dshowvideosrc_start (GstBaseSrc * bsrc)
   hres = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC,
       IID_IFilterGraph, (LPVOID *) & src->filter_graph);
   if (hres != S_OK || !src->filter_graph) {
-    GST_ERROR ("Can't create an instance of the dshow graph manager (error=0x%x)", hres);
+    GST_ERROR
+        ("Can't create an instance of the dshow graph manager (error=0x%x)",
+        hres);
     goto error;
   }
 
-  hres = src->filter_graph->QueryInterface(IID_IMediaFilter, 
-    (LPVOID *) &src->media_filter);
+  hres = src->filter_graph->QueryInterface (IID_IMediaFilter,
+      (LPVOID *) & src->media_filter);
   if (hres != S_OK || !src->media_filter) {
-    GST_ERROR ("Can't get IMediacontrol interface from the graph manager (error=0x%x)", hres);
+    GST_ERROR
+        ("Can't get IMediacontrol interface from the graph manager (error=0x%x)",
+        hres);
     goto error;
   }
 
   src->dshow_fakesink = new CDshowFakeSink;
-  src->dshow_fakesink->AddRef();
+  src->dshow_fakesink->AddRef ();
 
-  hres = src->filter_graph->AddFilter(src->video_cap_filter, L"capture");
+  hres = src->filter_graph->AddFilter (src->video_cap_filter, L"capture");
   if (hres != S_OK) {
-    GST_ERROR ("Can't add video capture filter to the graph (error=0x%x)", hres);
+    GST_ERROR ("Can't add video capture filter to the graph (error=0x%x)",
+        hres);
     goto error;
   }
 
-  hres = src->filter_graph->AddFilter(src->dshow_fakesink, L"sink");
+  hres = src->filter_graph->AddFilter (src->dshow_fakesink, L"sink");
   if (hres != S_OK) {
     GST_ERROR ("Can't add our fakesink filter to the graph (error=0x%x)", hres);
     goto error;
@@ -655,16 +650,16 @@ gst_dshowvideosrc_start (GstBaseSrc * bsrc)
 
 error:
   if (src->dshow_fakesink) {
-    src->dshow_fakesink->Release();
+    src->dshow_fakesink->Release ();
     src->dshow_fakesink = NULL;
   }
 
   if (src->media_filter) {
-    src->media_filter->Release();
+    src->media_filter->Release ();
     src->media_filter = NULL;
   }
   if (src->filter_graph) {
-    src->filter_graph->Release();
+    src->filter_graph->Release ();
     src->filter_graph = NULL;
   }
 
@@ -695,15 +690,11 @@ gst_dshowvideosrc_set_caps (GstBaseSrc * bsrc, GstCaps * caps)
 
     if (res != -1 && src->pins_mediatypes) {
       /* get the corresponding media type and build the dshow graph */
-      GList *type = g_list_nth (src->pins_mediatypes, res);
+      GList *type_pin_mediatype = g_list_nth (src->pins_mediatypes, res);
 
-      //will be removed when GST_TYPE_INT_RANGE_STEP exits
-      GList *type_video_default = g_list_nth (src->video_defaults, res);
-
-      if (type && type_video_default) {
-        //will be removed when GST_TYPE_INT_RANGE_STEP exits
-        GstCaptureVideoDefault *video_default = (GstCaptureVideoDefault *) type_video_default->data;
-        GstCapturePinMediaType *pin_mediatype = NULL;
+      if (type_pin_mediatype) {
+        GstCapturePinMediaType *pin_mediatype =
+            (GstCapturePinMediaType *) type_pin_mediatype->data;
         gchar *caps_string = NULL;
         gchar *src_caps_string = NULL;
 
@@ -718,31 +709,29 @@ gst_dshowvideosrc_set_caps (GstBaseSrc * bsrc, GstCaps * caps)
         gst_structure_get_fraction (s, "framerate", &numerator, &denominator);
 
         /* check if the desired video size is valid about granularity  */
-		    /* This check will be removed when GST_TYPE_INT_RANGE_STEP exits */
-        /* See remarks in gst_dshowvideosrc_getcaps_from_streamcaps function */
-        if (video_default->granularityWidth != 0 && width % video_default->granularityWidth != 0)
-          g_warning ("your desired video size is not valid : %d mod %d !=0\n", width, video_default->granularityWidth) ;
-        if (video_default->granularityHeight !=0 && height % video_default->granularityHeight != 0)
-          g_warning ("your desired video size is not valid : %d mod %d !=0\n", height, video_default->granularityHeight) ;
-
-        /* display all capabilities when using --gst-debug-level=3 */
-        src_caps_string = gst_caps_to_string (src->caps);
-        GST_LOG (src_caps_string);
-        g_free (src_caps_string);
-
-        pin_mediatype = (GstCapturePinMediaType *) type->data;
+        /* This check will be removed when GST_TYPE_INT_RANGE_STEP exits */
+        /* See remarks in gst_dshow_new_video_caps function */
+        if (pin_mediatype->granularityWidth != 0
+            && width % pin_mediatype->granularityWidth != 0)
+          g_warning ("your desired video size is not valid : %d mod %d !=0\n",
+              width, pin_mediatype->granularityWidth);
+        if (pin_mediatype->granularityHeight != 0
+            && height % pin_mediatype->granularityHeight != 0)
+          g_warning ("your desired video size is not valid : %d mod %d !=0\n",
+              height, pin_mediatype->granularityHeight);
 
         /* update mediatype */
         video_info = (VIDEOINFOHEADER *) pin_mediatype->mediatype->pbFormat;
         video_info->bmiHeader.biWidth = width;
         video_info->bmiHeader.biHeight = height;
-        video_info->AvgTimePerFrame = (LONGLONG) (10000000 * denominator / (double)numerator);
-        video_info->bmiHeader.biSizeImage = DIBSIZE(video_info->bmiHeader);
-        pin_mediatype->mediatype->lSampleSize = DIBSIZE(video_info->bmiHeader);
+        video_info->AvgTimePerFrame =
+            (LONGLONG) (10000000 * denominator / (double) numerator);
+        video_info->bmiHeader.biSizeImage = DIBSIZE (video_info->bmiHeader);
+        pin_mediatype->mediatype->lSampleSize = DIBSIZE (video_info->bmiHeader);
 
         src->dshow_fakesink->gst_set_media_type (pin_mediatype->mediatype);
-        src->dshow_fakesink->gst_set_buffer_callback(
-          (push_buffer_func) gst_dshowvideosrc_push_buffer, src);
+        src->dshow_fakesink->gst_set_buffer_callback (
+            (push_buffer_func) gst_dshowvideosrc_push_buffer, src);
 
         gst_dshow_get_pin_from_filter (src->dshow_fakesink, PINDIR_INPUT,
             &input_pin);
@@ -751,12 +740,14 @@ gst_dshowvideosrc_set_caps (GstBaseSrc * bsrc, GstCaps * caps)
           goto error;
         }
 
-        hres = src->filter_graph->ConnectDirect(pin_mediatype->capture_pin, 
-          input_pin, pin_mediatype->mediatype);
-        input_pin->Release();
+        hres = src->filter_graph->ConnectDirect (pin_mediatype->capture_pin,
+            input_pin, pin_mediatype->mediatype);
+        input_pin->Release ();
 
         if (hres != S_OK) {
-          GST_ERROR ("Can't connect capture filter with fakesink filter (error=0x%x)", hres);
+          GST_ERROR
+              ("Can't connect capture filter with fakesink filter (error=0x%x)",
+              hres);
           goto error;
         }
 
@@ -798,32 +789,44 @@ gst_dshowvideosrc_stop (GstBaseSrc * bsrc)
   gst_dshow_get_pin_from_filter (src->video_cap_filter, PINDIR_OUTPUT,
       &output_pin);
   if (output_pin) {
-    hres = src->filter_graph->Disconnect(output_pin);
-    output_pin->Release();
+    hres = src->filter_graph->Disconnect (output_pin);
+    output_pin->Release ();
   }
 
   gst_dshow_get_pin_from_filter (src->dshow_fakesink, PINDIR_INPUT, &input_pin);
   if (input_pin) {
-    hres = src->filter_graph->Disconnect(input_pin);
-    input_pin->Release();
+    hres = src->filter_graph->Disconnect (input_pin);
+    input_pin->Release ();
   }
 
-  /*remove filters from the graph */
-  src->filter_graph->RemoveFilter(src->video_cap_filter);
-  src->filter_graph->RemoveFilter(src->dshow_fakesink);
+  /* remove filters from the graph */
+  src->filter_graph->RemoveFilter (src->video_cap_filter);
+  src->filter_graph->RemoveFilter (src->dshow_fakesink);
 
-  /*release our gstreamer dshow sink */
-  src->dshow_fakesink->Release();
+  /* release our gstreamer dshow sink */
+  src->dshow_fakesink->Release ();
   src->dshow_fakesink = NULL;
 
-  /*release media filter interface */
-  src->media_filter->Release();
+  /* release media filter interface */
+  src->media_filter->Release ();
   src->media_filter = NULL;
 
-  /*release the filter graph manager */
-  src->filter_graph->Release();
+  /* release the filter graph manager */
+  src->filter_graph->Release ();
   src->filter_graph = NULL;
 
+  /* reset caps */
+  if (src->caps) {
+    gst_caps_unref (src->caps);
+    src->caps = NULL;
+  }
+
+  /* reset device id */
+  if (src->device) {
+    g_free (src->device);
+    src->device = NULL;
+  }
+  
   return TRUE;
 }
 
@@ -871,15 +874,14 @@ gst_dshowvideosrc_create (GstPushSrc * psrc, GstBuffer ** buf)
   }
 
   GST_DEBUG ("dshowvideosrc_create => pts %" GST_TIME_FORMAT " duration %"
-    GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (*buf)),
-    GST_TIME_ARGS (GST_BUFFER_DURATION (*buf)));
+      GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (*buf)),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (*buf)));
 
   return GST_FLOW_OK;
 }
 
 static GstCaps *
-gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc * src, IPin * pin,
-    IAMStreamConfig * streamcaps)
+gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc * src, IPin * pin)
 {
   GstCaps *caps = NULL;
   HRESULT hres = S_OK;
@@ -887,173 +889,113 @@ gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc * src, IPin * pin,
   int isize = 0;
   VIDEO_STREAM_CONFIG_CAPS vscc;
   int i = 0;
+  IAMStreamConfig *streamcaps = NULL;
 
-  if (!streamcaps)
+  hres = pin->QueryInterface (IID_IAMStreamConfig, (LPVOID *) & streamcaps);
+  if (FAILED (hres)) {
+    GST_ERROR ("Failed to retrieve IAMStreamConfig (error=0x%x)", hres);
     return NULL;
+  }
 
-  streamcaps->GetNumberOfCapabilities(&icount, &isize);
+  streamcaps->GetNumberOfCapabilities (&icount, &isize);
 
-  if (isize != sizeof (vscc))
+  if (isize != sizeof (vscc)) {
+    streamcaps->Release ();
     return NULL;
+  }
 
-  for (; i < icount; i++) {
-    GstCapturePinMediaType *pin_mediatype = g_new0 (GstCapturePinMediaType, 1);
-    GstCaptureVideoDefault *video_default = g_new0 (GstCaptureVideoDefault, 1);
+  caps = gst_caps_new_empty ();
 
-    pin->AddRef();
-    pin_mediatype->capture_pin = pin;
+  for (i = 0; i < icount; i++) {
+
+    GstCapturePinMediaType *pin_mediatype =
+      gst_dshow_new_pin_mediatype_from_streamcaps (pin, i, streamcaps);
+
+    if (pin_mediatype) {
 
-    hres = streamcaps->GetStreamCaps(i, &pin_mediatype->mediatype, (BYTE *) & vscc);
-    if (hres == S_OK && pin_mediatype->mediatype) {
-      VIDEOINFOHEADER *video_info;
       GstCaps *mediacaps = NULL;
+      GstVideoFormat video_format = 
+        gst_dshow_guid_to_gst_video_format (pin_mediatype->mediatype);
 
-      if (!caps)
-        caps = gst_caps_new_empty ();
+      if (video_format != GST_VIDEO_FORMAT_UNKNOWN) {
+        mediacaps = gst_dshow_new_video_caps (video_format, NULL,
+            pin_mediatype);
 
-      /* some remarks: */
-      /* Hope GST_TYPE_INT_RANGE_STEP will exits in future gstreamer releases  */
-      /* because we could use :  */
-      /* "width", GST_TYPE_INT_RANGE_STEP, video_default->minWidth, video_default->maxWidth,  video_default->granularityWidth */
-      /* instead of : */
-      /* "width", GST_TYPE_INT_RANGE, video_default->minWidth, video_default->maxWidth */
+      } else if (gst_dshow_check_mediatype (pin_mediatype->mediatype,
+              MEDIASUBTYPE_dvsd, FORMAT_VideoInfo)) {
+        mediacaps =
+            gst_dshow_new_video_caps (GST_VIDEO_FORMAT_UNKNOWN,
+            "video/x-dv, systemstream=FALSE", pin_mediatype);
 
-      /* For framerate we do not need a step (granularity) because  */
-      /* "The IAMStreamConfig::SetFormat method will set the frame rate to the closest  */
-      /* value that the filter supports" as it said in the VIDEO_STREAM_CONFIG_CAPS dshwo doc */
+      } else if (gst_dshow_check_mediatype (pin_mediatype->mediatype,
+              MEDIASUBTYPE_dvsd, FORMAT_DvInfo)) {
+        mediacaps =
+            gst_dshow_new_video_caps (GST_VIDEO_FORMAT_UNKNOWN,
+            "video/x-dv, systemstream=TRUE", pin_mediatype);
 
-      /* I420 */
-      if (gst_dshow_check_mediatype (pin_mediatype->mediatype, MEDIASUBTYPE_I420, FORMAT_VideoInfo)) {
-        video_info = (VIDEOINFOHEADER *) pin_mediatype->mediatype->pbFormat;
-
-        video_default->defaultWidth = video_info->bmiHeader.biWidth;
-        video_default->defaultHeight = video_info->bmiHeader.biHeight;
-        video_default->defaultFPS = (int) (10000000 / video_info->AvgTimePerFrame);
-        video_default->granularityWidth = vscc.OutputGranularityX;
-        video_default->granularityHeight = vscc.OutputGranularityY;
-
-        mediacaps = gst_caps_new_simple ("video/x-raw-yuv",
-          "width", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cx, vscc.MaxOutputSize.cx,
-          "height", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cy, vscc.MaxOutputSize.cy,
-          "framerate", GST_TYPE_FRACTION_RANGE,
-          (int) (10000000 / vscc.MaxFrameInterval), 1,
-          (int) (10000000 / vscc.MinFrameInterval), 1,
-          "format", GST_TYPE_FOURCC, MAKEFOURCC ('I', '4', '2', '0'), NULL);
-
-        if (mediacaps) {
-          src->pins_mediatypes =
-              g_list_append (src->pins_mediatypes, pin_mediatype);
-          src->video_defaults =
-            g_list_append (src->video_defaults, video_default);
-          gst_caps_append (caps, mediacaps);
-        } else {
-          gst_dshow_free_pin_mediatype (pin_mediatype);
-          g_free (video_default);
-        }
-        continue;
+        pin_mediatype->granularityWidth = 0;
+        pin_mediatype->granularityHeight = 0;
       }
 
-      /* BGR */
-      if (gst_dshow_check_mediatype (pin_mediatype->mediatype, MEDIASUBTYPE_RGB24, FORMAT_VideoInfo)) {
-        video_info = (VIDEOINFOHEADER *) pin_mediatype->mediatype->pbFormat;
-
-        video_default->defaultWidth = video_info->bmiHeader.biWidth;
-        video_default->defaultHeight = video_info->bmiHeader.biHeight;
-        video_default->defaultFPS = (int) (10000000 / video_info->AvgTimePerFrame);
-        video_default->granularityWidth = vscc.OutputGranularityX;
-        video_default->granularityHeight = vscc.OutputGranularityY;
-
-        /* ffmpegcolorspace handles RGB24 in BIG_ENDIAN */
-        mediacaps = gst_caps_new_simple ("video/x-raw-rgb",
-          "bpp", G_TYPE_INT, 24,
-          "depth", G_TYPE_INT, 24,
-          "width", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cx, vscc.MaxOutputSize.cx,
-          "height", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cy, vscc.MaxOutputSize.cy,
-          "framerate", GST_TYPE_FRACTION_RANGE,
-          (int) (10000000 / vscc.MaxFrameInterval), 1,
-          (int) (10000000 / vscc.MinFrameInterval), 1,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-          "red_mask", G_TYPE_INT, 255,
-          "green_mask", G_TYPE_INT, 65280,
-          "blue_mask", G_TYPE_INT, 16711680, NULL);
-
-        if (mediacaps) {
-          src->pins_mediatypes =
-              g_list_append (src->pins_mediatypes, pin_mediatype);
-          src->video_defaults =
-            g_list_append (src->video_defaults, video_default);
-          gst_caps_append (caps, mediacaps);
-        } else {
-          gst_dshow_free_pin_mediatype (pin_mediatype);
-          g_free (video_default);
-        }
-        continue;
+      if (mediacaps) {
+        src->pins_mediatypes =
+            g_list_append (src->pins_mediatypes, pin_mediatype);
+        gst_caps_append (caps, mediacaps);
+      } else {
+        /* failed to convert dshow caps */
+        gst_dshow_free_pin_mediatype (pin_mediatype);
       }
-
-      /* DVSD */
-      if (gst_dshow_check_mediatype (pin_mediatype->mediatype, MEDIASUBTYPE_dvsd, FORMAT_VideoInfo)) {
-        video_info = (VIDEOINFOHEADER *) pin_mediatype->mediatype->pbFormat;
-
-        video_default->defaultWidth = video_info->bmiHeader.biWidth;
-        video_default->defaultHeight = video_info->bmiHeader.biHeight;
-        video_default->defaultFPS = (int) (10000000 / video_info->AvgTimePerFrame);
-        video_default->granularityWidth = vscc.OutputGranularityX;
-        video_default->granularityHeight = vscc.OutputGranularityY;
-
-        mediacaps = gst_caps_new_simple ("video/x-dv",
-          "systemstream", G_TYPE_BOOLEAN, FALSE,
-          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('d', 'v', 's', 'd'),
-          "framerate", GST_TYPE_FRACTION_RANGE,
-          (int) (10000000 / vscc.MaxFrameInterval), 1,
-          (int) (10000000 / vscc.MinFrameInterval), 1,
-          "width", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cx, vscc.MaxOutputSize.cx,
-          "height", GST_TYPE_INT_RANGE, vscc.MinOutputSize.cy, vscc.MaxOutputSize.cy, NULL);
-
-        if (mediacaps) {
-          src->pins_mediatypes =
-              g_list_append (src->pins_mediatypes, pin_mediatype);
-          src->video_defaults =
-            g_list_append (src->video_defaults, video_default);
-          gst_caps_append (caps, mediacaps);
-        } else {
-          gst_dshow_free_pin_mediatype (pin_mediatype);
-          g_free (video_default);
-        }
-        continue;
-      }
-
-      /* DV stream */
-      if (gst_dshow_check_mediatype (pin_mediatype->mediatype, MEDIASUBTYPE_dvsd, FORMAT_DvInfo)) {
-        video_info = (VIDEOINFOHEADER *) pin_mediatype->mediatype->pbFormat;
-
-        //No video size in caps when stream ? I do know if the following fields exist
-        video_default->defaultWidth = video_info->bmiHeader.biWidth;
-        video_default->defaultHeight = video_info->bmiHeader.biHeight;
-        video_default->defaultFPS = (int) (10000000 / video_info->AvgTimePerFrame);
-        video_default->granularityWidth = vscc.OutputGranularityX;
-        video_default->granularityHeight = vscc.OutputGranularityY;
-
-        mediacaps = gst_caps_new_simple ("video/x-dv",
-            "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
-
-        if (mediacaps) {
-          src->pins_mediatypes =
-              g_list_append (src->pins_mediatypes, pin_mediatype);
-          src->video_defaults =
-            g_list_append (src->video_defaults, video_default);
-          gst_caps_append (caps, mediacaps);
-        } else {
-          gst_dshow_free_pin_mediatype (pin_mediatype);
-          g_free (video_default);
-        }
-        continue;
-      }
-    } else {
-      gst_dshow_free_pin_mediatype (pin_mediatype);
-      g_free (video_default);
     }
   }
 
+  streamcaps->Release ();
+
+  if (caps && gst_caps_is_empty (caps)) {
+    gst_caps_unref (caps);
+    caps = NULL;
+  }
+
+  return caps;
+}
+
+static GstCaps *
+gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc * src, IPin * pin)
+{
+  GstCaps *caps = NULL;
+  IEnumMediaTypes *enum_mediatypes = NULL;
+  HRESULT hres = S_OK;
+  GstCapturePinMediaType *pin_mediatype = NULL;
+
+  hres = pin->EnumMediaTypes (&enum_mediatypes);
+  if (FAILED (hres)) {
+    GST_ERROR ("Failed to retrieve IEnumMediaTypes (error=0x%x)", hres);
+    return NULL;
+  }
+
+  caps = gst_caps_new_empty ();
+
+  while ((pin_mediatype = gst_dshow_new_pin_mediatype_from_enum_mediatypes (pin, enum_mediatypes)) != NULL) {
+
+    GstCaps *mediacaps = NULL;
+    GstVideoFormat video_format = gst_dshow_guid_to_gst_video_format (pin_mediatype->mediatype);
+
+    if (video_format != GST_VIDEO_FORMAT_UNKNOWN)
+      mediacaps = gst_video_format_new_caps (video_format, 
+          pin_mediatype->defaultWidth, pin_mediatype->defaultHeight,
+          pin_mediatype->defaultFPS, 1, 1, 1);
+
+    if (mediacaps) {
+      src->pins_mediatypes =
+          g_list_append (src->pins_mediatypes, pin_mediatype);
+      gst_caps_append (caps, mediacaps);
+    } else {
+      /* failed to convert dshow caps */
+      gst_dshow_free_pin_mediatype (pin_mediatype);
+    }
+  }
+
+  enum_mediatypes->Release ();
+
   if (caps && gst_caps_is_empty (caps)) {
     gst_caps_unref (caps);
     caps = NULL;
@@ -1063,11 +1005,11 @@ gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc * src, IPin * pin,
 }
 
 static gboolean
-gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
-    UINT64 start, UINT64 stop)
+gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size, gpointer src_object,
+    GstClockTime duration)
 {
   GstDshowVideoSrc *src = GST_DSHOWVIDEOSRC (src_object);
-  GstBuffer *buf;
+  GstBuffer *buf = NULL;
   IPin *pPin = NULL;
   HRESULT hres = S_FALSE;
   AM_MEDIA_TYPE *pMediaType = NULL;
@@ -1080,12 +1022,16 @@ gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
   buf = gst_buffer_new_and_alloc (size);
 
   GST_BUFFER_SIZE (buf) = size;
-  GST_BUFFER_TIMESTAMP (buf) = gst_clock_get_time (GST_ELEMENT (src)->clock);
-  GST_BUFFER_TIMESTAMP (buf) -= GST_ELEMENT (src)->base_time;
-  GST_BUFFER_DURATION (buf) = stop - start;
+
+  GstClock *clock = gst_element_get_clock (GST_ELEMENT (src));
+  GST_BUFFER_TIMESTAMP (buf) =
+    GST_CLOCK_DIFF (gst_element_get_base_time (GST_ELEMENT (src)), gst_clock_get_time (clock));
+  gst_object_unref (clock);
+
+  GST_BUFFER_DURATION (buf) = duration;
 
   if (src->is_rgb) {
-    /* FOR RGB directshow decoder will return bottom-up BITMAP 
+    /* FOR RGB directshow decoder will return bottom-up BITMAP
      * There is probably a way to get top-bottom video frames from
      * the decoder...
      */
@@ -1100,8 +1046,9 @@ gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
     memcpy (GST_BUFFER_DATA (buf), buffer, size);
   }
 
-  GST_DEBUG ("push_buffer => pts %" GST_TIME_FORMAT "duration %" GST_TIME_FORMAT,
-    GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (stop - start));
+  GST_DEBUG ("push_buffer => pts %" GST_TIME_FORMAT "duration %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+      GST_TIME_ARGS (duration));
 
   /* the negotiate() method already set caps on the source pad */
   gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
